@@ -112,6 +112,13 @@ static uint64_t inflate_estimated_records(IntervUtils &iu, uint64_t estimated, d
 	return (uint64_t)inflated;
 }
 
+static bool allow_multichrom_range_split_for_gextract(SEXP iterator_policy, const string &intervset_out)
+{
+	// Keep legacy semantics for implicit iterators: splitting scope intervals across kids can
+	// change row geometry for iterator modes that depend on original interval boundaries.
+	return !Rf_isNull(iterator_policy) && intervset_out.empty();
+}
+
 static SEXP build_rintervals_extract(GIntervalsFetcher1D *out_intervals1d, GIntervalsFetcher2D *out_intervals2d, const vector< vector<double> > &values,
 									 vector<unsigned> *interv_ids, SEXP _exprs, SEXP _colnames, IntervUtils &iu)
 {
@@ -155,8 +162,55 @@ static SEXP build_rintervals_extract(GIntervalsFetcher1D *out_intervals1d, GInte
 		SET_STRING_ELT(col_names, num_interv_cols + num_exprs, Rf_mkChar("intervalID"));
 	}
     
-    runprotect(1); // col_names
-    return answer;
+	    runprotect(1); // col_names
+	    return answer;
+}
+
+static void unpack_kid_extract_result(
+	void *ptr,
+	uint64_t num_intervals,
+	bool is_1d_iterator,
+	unsigned num_exprs,
+	GIntervals &out_intervals1d,
+	GIntervals2D &out_intervals2d,
+	SEXP ids,
+	const vector<SEXP> &r_expr_vals,
+	uint64_t values_offset)
+{
+	if (!num_intervals)
+		return;
+
+	if (is_1d_iterator) {
+		out_intervals1d.insert(out_intervals1d.end(), (GInterval *)ptr, (GInterval *)ptr + num_intervals);
+		ptr = (GInterval *)ptr + num_intervals;
+	} else {
+		out_intervals2d.insert(out_intervals2d.end(), (GInterval2D *)ptr, (GInterval2D *)ptr + num_intervals);
+		ptr = (GInterval2D *)ptr + num_intervals;
+	}
+
+	unsigned *kid_ids = (unsigned *)ptr;
+	for (uint64_t iid = 0; iid < num_intervals; ++iid)
+		INTEGER(ids)[values_offset + iid] = kid_ids[iid];
+	ptr = kid_ids + num_intervals;
+
+	for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
+		double *kid_vals = (double *)ptr;
+		memcpy(REAL(r_expr_vals[iexpr]) + values_offset, kid_vals, sizeof(double) * num_intervals);
+		ptr = kid_vals + num_intervals;
+	}
+}
+
+static void set_extract_value_column_names(SEXP answer, unsigned num_interv_cols, unsigned num_exprs, SEXP _exprs, SEXP _colnames)
+{
+	SEXP col_names = rprotect_ptr(Rf_getAttrib(answer, R_NamesSymbol));
+	for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
+		if (Rf_isNull(_colnames))
+			SET_STRING_ELT(col_names, num_interv_cols + iexpr, Rf_mkChar(get_bounded_colname(CHAR(STRING_ELT(_exprs, iexpr))).c_str()));
+		else
+			SET_STRING_ELT(col_names, num_interv_cols + iexpr, STRING_ELT(_colnames, iexpr));
+	}
+	SET_STRING_ELT(col_names, num_interv_cols + num_exprs, Rf_mkChar("intervalID"));
+	runprotect(1); // col_names
 }
 
 
@@ -528,7 +582,11 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 			return R_NilValue;
 		}
 
-		if (!iu.prepare4multitasking(_exprs, intervals1d, intervals2d, _iterator_policy, _band))
+		// IMPORTANT: do not add estimated-size mode gating here.
+		// We intentionally rely on prepare4multitasking() + runtime allocation guards,
+		// because estimate-based auto-disabling previously regressed gextract throughput.
+		bool allow_multichrom_1d_range_split = allow_multichrom_range_split_for_gextract(_iterator_policy, intervset_out);
+		if (!iu.prepare4multitasking(_exprs, intervals1d, intervals2d, _iterator_policy, _band, allow_multichrom_1d_range_split))
 			rreturn(R_NilValue);
 
 		bool is_1d_iterator = iu.is_1d_iterator(_exprs, intervals1d, intervals2d, _iterator_policy);
@@ -664,14 +722,16 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 												  sizeof(double) * num_exprs);                                     // values
 				}
 				break;
-			} catch (TGLException &e) {
-				string msg(e.msg());
-				if (msg.find("Failed to allocate shared memory") != string::npos) {
-					if (base_estimated_records > 0 && max_records_factor > 1.0) {
-						max_records_factor = max(1.0, max_records_factor / 2.0);
-						estimated_records = inflate_estimated_records(iu, base_estimated_records, max_records_factor);
-						continue;
-					}
+				} catch (TGLException &e) {
+					string msg(e.msg());
+					if (msg.find("Failed to allocate shared memory") != string::npos) {
+						// Fallback policy: keep multitasking unless shared-memory allocation fails.
+						// We first back off the inflation factor; only then fall back to serial.
+						if (base_estimated_records > 0 && max_records_factor > 1.0) {
+							max_records_factor = max(1.0, max_records_factor / 2.0);
+							estimated_records = inflate_estimated_records(iu, base_estimated_records, max_records_factor);
+							continue;
+						}
 					return C_gextract(_intervals, _exprs, _colnames, _iterator_policy, _band, R_NilValue, _intervals_set_out, _envir);
 				}
 				throw;
@@ -734,49 +794,37 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 				log_gextract_timing("child_scan_pack_ms", scan_ms);
 			}
 			
-		} else {  // parent process
-			auto t_gather_start = std::chrono::steady_clock::now();
+			} else {  // parent process
+				auto t_gather_start = std::chrono::steady_clock::now();
 
-			// collect results from kids
-			// First pass: compute total sizes to pre-reserve and avoid repeated reallocations.
-			uint64_t total_intervals = 0;
-			for (int i = 0; i < get_num_kids(); ++i) {
-				total_intervals += get_kid_res_size(i);
-			}
-			if (is_1d_iterator)
-				out_intervals1d.reserve(total_intervals);
-			else
-				out_intervals2d.reserve(total_intervals);
-			interv_ids.reserve(total_intervals);
-			for (unsigned i = 0; i < num_exprs; ++i)
-				values[i].reserve(total_intervals);
+				// collect results from kids
+				// First pass: compute total sizes to pre-reserve and avoid repeated reallocations.
+				uint64_t total_intervals = 0;
+				for (int i = 0; i < get_num_kids(); ++i) {
+					total_intervals += get_kid_res_size(i);
+				}
+				if (is_1d_iterator)
+					out_intervals1d.reserve(total_intervals);
+				else
+					out_intervals2d.reserve(total_intervals);
 
-			for (int i = 0; i < get_num_kids(); ++i) {
-				void *ptr = get_kid_res(i);
-				num_intervals = get_kid_res_size(i);
+				vector<SEXP> r_expr_vals(num_exprs, R_NilValue);
+				for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr)
+					r_expr_vals[iexpr] = rprotect_ptr(RSaneAllocVector(REALSXP, total_intervals));
+				SEXP ids = rprotect_ptr(RSaneAllocVector(INTSXP, total_intervals));
+				uint64_t values_offset = 0;
 
-				if (!num_intervals)
-					continue;
+				for (int i = 0; i < get_num_kids(); ++i) {
+					void *ptr = get_kid_res(i);
+					num_intervals = get_kid_res_size(i);
 
-				if (is_1d_iterator) {
-					out_intervals1d.insert(out_intervals1d.end(), (GInterval *)ptr, (GInterval *)ptr + num_intervals);
-					ptr = (GInterval *)ptr + num_intervals;
-				} else {
-					out_intervals2d.insert(out_intervals2d.end(), (GInterval2D *)ptr, (GInterval2D *)ptr + num_intervals);
-					ptr = (GInterval2D *)ptr + num_intervals;
+					unpack_kid_extract_result(ptr, num_intervals, is_1d_iterator, num_exprs,
+						out_intervals1d, out_intervals2d, ids, r_expr_vals, values_offset);
+					values_offset += num_intervals;
 				}
 
-				interv_ids.insert(interv_ids.end(), (unsigned *)ptr, (unsigned *)ptr + num_intervals);
-				ptr = (unsigned *)ptr + num_intervals;
-
-				for (unsigned i = 0; i < num_exprs; ++i) {
-					values[i].insert(values[i].end(), (double *)ptr, (double *)ptr + num_intervals);
-					ptr = (double *)ptr + num_intervals;
-				}
-			}
-
-			if (out_intervals1d.empty() && out_intervals2d.empty()) 
-				rreturn(R_NilValue);
+				if (out_intervals1d.empty() && out_intervals2d.empty()) 
+					rreturn(R_NilValue);
 
 			auto t_assemble_start = std::chrono::steady_clock::now();
 
@@ -792,29 +840,12 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 				num_interv_cols = GInterval2D::NUM_COLS;
 			}
 
-            for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
-                SEXP expr_vals = rprotect_ptr(RSaneAllocVector(REALSXP, values[iexpr].size()));
-				// values[iexpr] is contiguous; copy in one shot
-				if (!values[iexpr].empty())
-					memcpy(REAL(expr_vals), &values[iexpr][0], sizeof(double) * values[iexpr].size());
-                SET_VECTOR_ELT(answer, num_interv_cols + iexpr, expr_vals);
-			}
-
-            SEXP ids = rprotect_ptr(RSaneAllocVector(INTSXP, interv_ids.size()));
-			if (!interv_ids.empty())
-				memcpy(INTEGER(ids), &interv_ids[0], sizeof(int) * interv_ids.size());
+			for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr)
+				SET_VECTOR_ELT(answer, num_interv_cols + iexpr, r_expr_vals[iexpr]);
 			SET_VECTOR_ELT(answer, num_interv_cols + num_exprs, ids);
 
-            SEXP col_names = rprotect_ptr(Rf_getAttrib(answer, R_NamesSymbol));
-			for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
-				if (Rf_isNull(_colnames))
-					SET_STRING_ELT(col_names, num_interv_cols + iexpr, Rf_mkChar(get_bounded_colname(CHAR(STRING_ELT(_exprs, iexpr))).c_str()));
-				else
-					SET_STRING_ELT(col_names, num_interv_cols + iexpr, STRING_ELT(_colnames, iexpr));
-			}
-			SET_STRING_ELT(col_names, num_interv_cols + num_exprs, Rf_mkChar("intervalID"));
-
-            runprotect(2); // col_names, ids
+			set_extract_value_column_names(answer, num_interv_cols, num_exprs, _exprs, _colnames);
+			runprotect(1); // ids
 
 			if (profile) {
 				auto t_end = std::chrono::steady_clock::now();
