@@ -10,11 +10,13 @@
 
 #include <cstdint>
 #include <cmath>
+#include <deque>
 #include <limits>
 #include <string>
 #include <vector>
 
 #include "GenomeTrack1D.h"
+#include "utils/RunningLogSumExp.h"
 
 // !!!!!!!!! IN CASE OF ERROR THIS CLASS THROWS TGLException  !!!!!!!!!!!!!!!!
 
@@ -25,6 +27,7 @@ public:
 	void read_interval(const GInterval &interval) override;
 	double last_max_pos() const override;
 	double last_min_pos() const override;
+	void set_master_obj(GenomeTrackFixedBin *master_obj) { m_master_obj = master_obj; m_master_synced = false; }
 
 	void init_read(const char *filename, int chromid) { init_read(filename, "rb", chromid); }
 	void init_write(const char *filename, unsigned bin_size, int chromid);
@@ -61,6 +64,29 @@ protected:
 
 	// Helper to parse header at current file position
 	void read_header_at_current_pos_(BufferedFile &bf);
+
+	// Sliding window LSE state
+	RunningLogSumExp m_running_lse;
+	std::deque<float> m_lse_window_bins;
+	int64_t m_lse_prev_sbin{-1};
+	int64_t m_lse_prev_ebin{-1};
+	double m_sliding_sum{0.0};
+	int64_t m_sliding_num_vs{0};
+	bool m_lse_sliding_valid{false};
+	bool m_running_lse_initialized{false};
+	int m_fast_path_mode{0}; // 0=unknown, 1=reducer-only fast path, 2=avg/nearest-only fast path, -1=generic path
+	uint32_t m_fast_reducer_bits{0};
+	GenomeTrackFixedBin *m_master_obj{NULL};
+	bool m_master_synced{false};
+
+	void read_interval_reducers_only(const GInterval &interval);
+	void read_interval_avg_nearest_only(const GInterval &interval);
+	void sync_master_state_from_dependent();
+	void copy_state_from_master();
+	void classify_fast_path_mode();
+	void assign_single_bin_value(float value, double overlap_start);
+	void assign_single_bin_missing();
+	void reset_sliding_window_state();
 };
 
 
