@@ -6,6 +6,7 @@
  */
 
 #include <cstdint>
+#include <limits>
 #include <string>
 #include <vector>
 
@@ -14,10 +15,86 @@
 #include "rdbinterval.h"
 #include "rdbutils.h"
 #include "TrackExpressionScanner.h"
+#include "TrackExpressionFixedBinIterator.h"
+#include "TrackExpressionFixedRectIterator.h"
+#include "TrackExpressionIntervals1DIterator.h"
+#include "TrackExpressionSparseIterator.h"
+#include "TGLException.h"
 
 using namespace std;
 using namespace rdb;
 
+static uint64_t estimate_records_for_expr(
+	IntervUtils &iu,
+	SEXP expr,
+	GIntervalsFetcher1D *scope1d,
+	GIntervalsFetcher2D *scope2d,
+	SEXP iterator_policy,
+	SEXP band)
+{
+	uint64_t estimated = iu.estimate_num_bins(iterator_policy, scope1d, scope2d);
+	if (estimated)
+		return estimated;
+
+	TrackExprScanner scanner(iu);
+	TrackExpressionIteratorBase *expr_itr = scanner.create_expr_iterator(expr, scope1d, scope2d, iterator_policy, band, true);
+
+	if (auto *bin_itr = dynamic_cast<TrackExpressionFixedBinIterator *>(expr_itr)) {
+		int64_t binsize = bin_itr->get_bin_size();
+		if (binsize > 0) {
+			SEXP rbinsize = PROTECT(Rf_ScalarReal((double)binsize));
+			estimated = iu.estimate_num_bins(rbinsize, scope1d, scope2d);
+			UNPROTECT(1);
+			return estimated;
+		}
+	}
+
+	if (auto *rect_itr = dynamic_cast<TrackExpressionFixedRectIterator *>(expr_itr)) {
+		int64_t width = rect_itr->get_width();
+		int64_t height = rect_itr->get_height();
+		if (width > 0 && height > 0) {
+			SEXP rrect = PROTECT(Rf_allocVector(REALSXP, 2));
+			REAL(rrect)[0] = (double)width;
+			REAL(rrect)[1] = (double)height;
+			estimated = iu.estimate_num_bins(rrect, scope1d, scope2d);
+			UNPROTECT(1);
+			return estimated;
+		}
+	}
+
+	if (dynamic_cast<TrackExpressionSparseIterator *>(expr_itr)) {
+		return 0;
+	}
+
+	// Intervals iterators can emit more records than the number of iterator intervals
+	// when a long iterator interval overlaps multiple scope intervals.
+	// Avoid underestimating shared-memory size by not capping in this case.
+	if (dynamic_cast<TrackExpressionIntervals1DIterator *>(expr_itr)) {
+		return 0;
+	}
+
+	return 0;
+}
+
+static uint64_t inflate_estimated_records(IntervUtils &iu, uint64_t estimated, double factor)
+{
+	if (!estimated)
+		return 0;
+
+	if (!(factor > 1.0))
+		return estimated;
+
+	long double inflated = (long double)estimated * factor;
+	long double max_records = (long double)iu.get_max_data_size();
+
+	if (inflated > max_records)
+		return (uint64_t)max_records;
+	if (inflated > (long double)std::numeric_limits<uint64_t>::max())
+		return std::numeric_limits<uint64_t>::max();
+
+	return (uint64_t)inflated;
+}
+
 #ifndef R_NO_REMAP
 #  define R_NO_REMAP
 #endif
@@ -167,9 +244,9 @@ SEXP C_gscreen(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP _band, S
 		}
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-    } catch (const bad_alloc &e) {
-        rerror("Out of memory");
-    }
+	} catch (const bad_alloc &e) {
+		rerror("Out of memory");
+	}
 	return R_NilValue;
 }
 
@@ -204,16 +281,10 @@ SEXP gscreen_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 		bool is_1d_iterator = iu.is_1d_iterator(_expr, intervals1d, intervals2d, _iterator_policy);
 		vector<GIntervalsBigSet1D::ChromStat> chromstats1d;
 		vector<GIntervalsBigSet2D::ChromStat> chromstats2d;
-
-		// Estimate result size - gscreen is non-deterministic (depends on filter)
-		// Use conservative estimate: assume all intervals might pass
-		uint64_t estimated_records = is_1d_iterator ? intervals1d->size() : intervals2d->size();
-		rdb::MultitaskingMode mode = iu.select_multitasking_mode(false, estimated_records);
-
-		// If mode is SINGLE (size too large), fall back to non-multitasking version
-		if (mode == rdb::MT_MODE_SINGLE) {
-			return C_gscreen(_expr, _intervals, _iterator_policy, _band, _intervals_set_out, _envir);
-		}
+		// Estimate number of records to cap shared-memory allocation size
+		uint64_t base_estimated_records = estimate_records_for_expr(iu, _expr, intervals1d, intervals2d, _iterator_policy, _band);
+		double max_records_factor = iu.get_multitask_max_records_factor();
+		uint64_t estimated_records = inflate_estimated_records(iu, base_estimated_records, max_records_factor);
 
 		if (!intervset_out.empty()) {
 			if (is_1d_iterator)
@@ -222,10 +293,41 @@ SEXP gscreen_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 				GIntervalsBigSet2D::begin_save(intervset_out.c_str(), iu, chromstats2d);
 		}
 
-		if ((intervset_out.empty() && iu.distribute_task(0, (is_1d_iterator ? sizeof(GInterval) : sizeof(GInterval2D)))) ||
-			(!intervset_out.empty() && iu.distribute_task(is_1d_iterator ?
-														 sizeof(GIntervalsBigSet1D::ChromStat) * chromstats1d.size() :
-														 sizeof(GIntervalsBigSet2D::ChromStat) * chromstats2d.size(), 0)) )
+		bool do_child = false;
+		for (;;) {
+			try {
+				if (intervset_out.empty()) {
+					if (estimated_records > 0) {
+						do_child = iu.distribute_task(0,
+													  (is_1d_iterator ? sizeof(GInterval) : sizeof(GInterval2D)),
+													  rdb::MT_MODE_MMAP,
+													  estimated_records);
+					} else {
+						do_child = iu.distribute_task(0,
+													  (is_1d_iterator ? sizeof(GInterval) : sizeof(GInterval2D)));
+					}
+				} else {
+					do_child = iu.distribute_task(is_1d_iterator ?
+												  sizeof(GIntervalsBigSet1D::ChromStat) * chromstats1d.size() :
+												  sizeof(GIntervalsBigSet2D::ChromStat) * chromstats2d.size(),
+												  0);
+				}
+				break;
+			} catch (TGLException &e) {
+				string msg(e.msg());
+				if (msg.find("Failed to allocate shared memory") != string::npos) {
+					if (base_estimated_records > 0 && max_records_factor > 1.0) {
+						max_records_factor = max(1.0, max_records_factor / 2.0);
+						estimated_records = inflate_estimated_records(iu, base_estimated_records, max_records_factor);
+						continue;
+					}
+					return C_gscreen(_expr, _intervals, _iterator_policy, _band, _intervals_set_out, _envir);
+				}
+				throw;
+			}
+		}
+
+		if (do_child)
 		{ // child process
 			TrackExprScanner scanner(iu);
 
@@ -346,10 +448,14 @@ SEXP gscreen_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 			}
 		}
 	} catch (TGLException &e) {
+		string msg(e.msg());
+		if (msg.find("Result size exceeded the maximal allowed") != string::npos) {
+			return C_gscreen(_expr, _intervals, _iterator_policy, _band, _intervals_set_out, _envir);
+		}
 		rerror("%s", e.msg());
-    } catch (const bad_alloc &e) {
-        rerror("Out of memory");
-    }
+	} catch (const bad_alloc &e) {
+		rerror("Out of memory");
+	}
 	rreturn(R_NilValue);
 }
 
