@@ -1,7 +1,6 @@
 #include <cstdint>
 #include <cmath>
 #include <unistd.h>
-#include <unordered_map>
 #include <unordered_set>
 
 #include "rdbutils.h"
@@ -27,7 +26,7 @@
 #include "SequenceVarProcessor.h"
 
 const char *TrackExpressionVars::Track_var::FUNC_NAMES[TrackExpressionVars::Track_var::NUM_FUNCS] = {
-	"avg", "min", "max", "nearest", "stddev", "sum", "quantile",
+	"avg", "min", "max", "nearest", "stddev", "sum", "lse", "quantile",
 	"global.percentile", "global.percentile.min", "global.percentile.max",
 	"weighted.sum", "area", "pwm", "pwm.max", "pwm.max.pos", "pwm.count", "kmer.count", "kmer.frac",
     "masked.count", "masked.frac",
@@ -38,7 +37,7 @@ const char *TrackExpressionVars::Track_var::FUNC_NAMES[TrackExpressionVars::Trac
 const char *TrackExpressionVars::Interv_var::FUNC_NAMES[TrackExpressionVars::Interv_var::NUM_FUNCS] = { "distance", "distance.center", "distance.edge", "coverage", "neighbor.count" };
 
 const char *TrackExpressionVars::Value_var::FUNC_NAMES[TrackExpressionVars::Value_var::NUM_FUNCS] = {
-	"avg", "min", "max", "stddev", "sum", "quantile",
+	"avg", "min", "max", "stddev", "sum", "lse", "quantile",
 	"nearest",
 	"exists", "size",
 	"first", "last", "sample",
@@ -51,6 +50,73 @@ const char *TrackExpressionVars::Value_var::FUNC_NAMES[TrackExpressionVars::Valu
 
 using namespace rdb;
 
+namespace {
+shared_ptr<GenomeTrack> create_and_init_1d_track(const string &filename, int chromid, GenomeTrack::Type type)
+{
+	shared_ptr<GenomeTrack> track;
+
+	if (type == GenomeTrack::FIXED_BIN) {
+		auto t = make_shared<GenomeTrackFixedBin>();
+		t->init_read(filename.c_str(), chromid);
+		track = t;
+	} else if (type == GenomeTrack::SPARSE) {
+		auto t = make_shared<GenomeTrackSparse>();
+		t->init_read(filename.c_str(), chromid);
+		track = t;
+	} else if (type == GenomeTrack::ARRAYS) {
+		auto t = make_shared<GenomeTrackArrays>();
+		t->init_read(filename.c_str(), chromid);
+		track = t;
+	}
+
+	return track;
+}
+
+bool supports_shared_1d_backend(GenomeTrack::Type type)
+{
+	return type == GenomeTrack::FIXED_BIN || type == GenomeTrack::SPARSE;
+}
+
+bool is_shared_1d_backend_enabled()
+{
+	SEXP opt = Rf_GetOption1(Rf_install("gshared.track.backend"));
+	if (Rf_isNull(opt))
+		return false;
+	return Rf_asLogical(opt) == 1;
+}
+
+shared_ptr<GenomeTrack> create_dependent_1d_track(GenomeTrack::Type type, const shared_ptr<GenomeTrack> &master_track)
+{
+	if (type == GenomeTrack::FIXED_BIN) {
+		GenomeTrackFixedBin *master = dynamic_cast<GenomeTrackFixedBin *>(master_track.get());
+		if (!master)
+			return shared_ptr<GenomeTrack>();
+
+		auto dependent = make_shared<GenomeTrackFixedBin>();
+		dependent->set_master_obj(master);
+		return dependent;
+	}
+
+	if (type == GenomeTrack::SPARSE) {
+		GenomeTrackSparse *master = dynamic_cast<GenomeTrackSparse *>(master_track.get());
+		if (!master)
+			return shared_ptr<GenomeTrack>();
+
+		auto dependent = make_shared<GenomeTrackSparse>();
+		dependent->set_master_obj(master);
+		return dependent;
+	}
+
+	return shared_ptr<GenomeTrack>();
+}
+
+string shared_1d_backend_key(GenomeTrack::Type type, const string &filename, int chromid)
+{
+	return to_string((int)type) + ":" + to_string(chromid) + ":" + filename;
+}
+
+}  // namespace
+
 
 TrackExpressionVars::TrackExpressionVars(rdb::IntervUtils &iu) :
 	m_iu(iu)
@@ -74,8 +140,6 @@ TrackExpressionVars::~TrackExpressionVars()
 		runprotect(ivar->rvar);
 	for (Value_vars::iterator ivar = m_value_vars.begin(); ivar != m_value_vars.end(); ivar++)
 		runprotect(ivar->rvar);
-	for (Track_n_imdfs::iterator itrack_n_imdf = m_track_n_imdfs.begin(); itrack_n_imdf != m_track_n_imdfs.end(); ++itrack_n_imdf)
-		delete itrack_n_imdf->track;
 }
 
 void TrackExpressionVars::parse_exprs(const vector<string> &track_exprs)
@@ -294,7 +358,7 @@ TrackExpressionVars::Track_n_imdf &TrackExpressionVars::add_track_n_imdf(const s
 	m_track_n_imdfs.push_back(Track_n_imdf());
 	Track_n_imdf &track_n_imdf = m_track_n_imdfs.back();
 	track_n_imdf.name = track;
-	track_n_imdf.track = NULL;
+	track_n_imdf.track.reset();
 	track_n_imdf.type = track_type;
 	track_n_imdf.slice = slice;
 	track_n_imdf.slice_func = slice_func;
@@ -695,7 +759,7 @@ TrackExpressionVars::Track_var &TrackExpressionVars::add_vtrack_var_src_track(SE
 	for (ifunc = 0; ifunc < Track_var::NUM_FUNCS; ++ifunc) {
 		if (!strcmp(func.c_str(), Track_var::FUNC_NAMES[ifunc])) {
 			if ((GenomeTrack::is_1d(track_type) && (ifunc == Track_var::WEIGHTED_SUM || ifunc == Track_var::OCCUPIED_AREA)) ||
-					(GenomeTrack::is_2d(track_type) && (ifunc == Track_var::REG_NEAREST || ifunc == Track_var::STDDEV || ifunc == Track_var::SUM || ifunc == Track_var::QUANTILE)) ||
+					(GenomeTrack::is_2d(track_type) && (ifunc == Track_var::REG_NEAREST || ifunc == Track_var::STDDEV || ifunc == Track_var::SUM || ifunc == Track_var::LSE || ifunc == Track_var::QUANTILE)) ||
 					(track_type != GenomeTrack::FIXED_BIN && (ifunc == Track_var::PV || ifunc == Track_var::PV_MIN || ifunc == Track_var::PV_MAX)))
 				verror("Virtual track %s: function %s is not supported by %s tracks", vtrack.c_str(), func.c_str(), GenomeTrack::TYPE_NAMES[track_type]);
 	
@@ -1059,6 +1123,9 @@ TrackExpressionVars::Value_var &TrackExpressionVars::add_vtrack_var_src_value(SE
 		case Value_var::SUM:
 			var.track->register_function(GenomeTrack1D::SUM);
 			break;
+		case Value_var::LSE:
+			var.track->register_function(GenomeTrack1D::LSE);
+			break;
 		case Value_var::STDDEV:
 			var.track->register_function(GenomeTrack1D::STDDEV);
 			var.track->register_function(GenomeTrack1D::SIZE);
@@ -1123,8 +1190,8 @@ void TrackExpressionVars::register_track_functions()
         if (TrackExpressionVars::is_sequence_based_function(ivar->val_func)) {
             continue;
         }
-		GenomeTrack1D *track1d = GenomeTrack::is_1d(ivar->track_n_imdf->type) ? (GenomeTrack1D *)ivar->track_n_imdf->track : NULL;
-		GenomeTrack2D *track2d = GenomeTrack::is_2d(ivar->track_n_imdf->type) ? (GenomeTrack2D *)ivar->track_n_imdf->track : NULL;
+		GenomeTrack1D *track1d = GenomeTrack::is_1d(ivar->track_n_imdf->type) ? (GenomeTrack1D *)ivar->track_n_imdf->track.get() : NULL;
+		GenomeTrack2D *track2d = GenomeTrack::is_2d(ivar->track_n_imdf->type) ? (GenomeTrack2D *)ivar->track_n_imdf->track.get() : NULL;
 
 		switch (ivar->val_func) {
 		case Track_var::REG:
@@ -1169,6 +1236,11 @@ void TrackExpressionVars::register_track_functions()
 		case Track_var::SUM:
 			track1d->register_function(GenomeTrack1D::SUM);
 			break;
+		case Track_var::LSE:
+			if (!track1d)
+				verror("vtrack function 'lse' can only be used on 1D tracks");
+			track1d->register_function(GenomeTrack1D::LSE);
+			break;
 		case Track_var::QUANTILE:
 			track1d->register_quantile(m_iu.get_max_data_size(), m_iu.get_quantile_edge_data_size(), m_iu.get_quantile_edge_data_size());
 			break;
@@ -1228,11 +1300,11 @@ void TrackExpressionVars::register_track_functions()
 			break;
 		}
 
-		if (ivar->track_n_imdf->type == GenomeTrack::ARRAYS) {
-			GenomeTrackArrays *track = (GenomeTrackArrays *)ivar->track_n_imdf->track;
-			if (ivar->track_n_imdf->slice_func == GenomeTrackArrays::S_QUANTILE) 
-				track->set_slice_quantile(ivar->track_n_imdf->slice_percentile, m_iu.get_max_data_size(), m_iu.get_quantile_edge_data_size(), m_iu.get_quantile_edge_data_size(),
-										  ivar->track_n_imdf->slice);
+			if (ivar->track_n_imdf->type == GenomeTrack::ARRAYS) {
+				GenomeTrackArrays *track = (GenomeTrackArrays *)ivar->track_n_imdf->track.get();
+				if (ivar->track_n_imdf->slice_func == GenomeTrackArrays::S_QUANTILE) 
+					track->set_slice_quantile(ivar->track_n_imdf->slice_percentile, m_iu.get_max_data_size(), m_iu.get_quantile_edge_data_size(), m_iu.get_quantile_edge_data_size(),
+											  ivar->track_n_imdf->slice);
 			else 
 				track->set_slice_function(ivar->track_n_imdf->slice_func, ivar->track_n_imdf->slice);
 		}
@@ -1422,22 +1494,54 @@ void TrackExpressionVars::define_r_vars(unsigned size)
 	}
 }
 
+bool TrackExpressionVars::is_sequence_track_n_imdf(const Track_n_imdf &track_n_imdf) const
+{
+	for (Track_vars::const_iterator ivar = m_track_vars.begin(); ivar != m_track_vars.end(); ++ivar) {
+		if (ivar->track_n_imdf == &track_n_imdf &&
+			TrackExpressionVars::is_sequence_based_function(ivar->val_func))
+			return true;
+	}
+
+	return false;
+}
+
+void TrackExpressionVars::reset_shared_1d_track_masters()
+{
+	m_shared_1d_track_masters.clear();
+}
+
+shared_ptr<GenomeTrack> TrackExpressionVars::init_1d_track_with_shared_backend(const string &filename, int chromid, GenomeTrack::Type type)
+{
+	if (!is_shared_1d_backend_enabled() || !supports_shared_1d_backend(type))
+		return create_and_init_1d_track(filename, chromid, type);
+
+	const string backend_key = shared_1d_backend_key(type, filename, chromid);
+
+	shared_ptr<GenomeTrack> master_track;
+	unordered_map<string, shared_ptr<GenomeTrack> >::iterator imaster = m_shared_1d_track_masters.find(backend_key);
+	if (imaster == m_shared_1d_track_masters.end()) {
+		master_track = create_and_init_1d_track(filename, chromid, type);
+		if (!master_track)
+			return shared_ptr<GenomeTrack>();
+		m_shared_1d_track_masters.insert(make_pair(backend_key, master_track));
+	} else
+		master_track = imaster->second;
+
+	shared_ptr<GenomeTrack> dependent_track = create_dependent_1d_track(type, master_track);
+	return dependent_track ? dependent_track : master_track;
+}
+
 void TrackExpressionVars::start_chrom(const GInterval &interval)
 {
+	reset_shared_1d_track_masters();
+
+	// Precompute sequence-only flags (avoids O(N*M) scan in set_vars hot loop)
+	for (Track_n_imdfs::iterator itn = m_track_n_imdfs.begin(); itn != m_track_n_imdfs.end(); ++itn)
+		itn->has_only_sequence_functions = is_sequence_track_n_imdf(*itn);
+
 	for (Track_n_imdfs::iterator itrack_n_imdf = m_track_n_imdfs.begin(); itrack_n_imdf != m_track_n_imdfs.end(); ++itrack_n_imdf)
 	{
-		// Skip track initialization for sequence-based tracks
-		bool is_sequence_track = false;
-		for (Track_vars::iterator ivar = m_track_vars.begin(); ivar != m_track_vars.end(); ++ivar)
-		{
-			if (ivar->track_n_imdf == &(*itrack_n_imdf) &&
-				TrackExpressionVars::is_sequence_based_function(ivar->val_func))
-			{
-				is_sequence_track = true;
-				break;
-			}
-		}
-		if (is_sequence_track)
+		if (itrack_n_imdf->has_only_sequence_functions)
 			continue;
 
 		try
@@ -1445,27 +1549,12 @@ void TrackExpressionVars::start_chrom(const GInterval &interval)
 			string track_dir = track2path(m_iu.get_env(), itrack_n_imdf->name);
 			string resolved = GenomeTrack::find_existing_1d_filename(m_iu.get_chromkey(), track_dir, interval.chromid);
 			string filename(track_dir + "/" + resolved);
-
-			delete itrack_n_imdf->track;
-			if (itrack_n_imdf->type == GenomeTrack::FIXED_BIN)
-			{
-				itrack_n_imdf->track = new GenomeTrackFixedBin;
-				((GenomeTrackFixedBin *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
-			}
-			else if (itrack_n_imdf->type == GenomeTrack::SPARSE)
-			{
-				itrack_n_imdf->track = new GenomeTrackSparse;
-				((GenomeTrackSparse *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
-			}
-			else if (itrack_n_imdf->type == GenomeTrack::ARRAYS)
-			{
-				itrack_n_imdf->track = new GenomeTrackArrays;
-				((GenomeTrackArrays *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
-			}
-			else
-			{
-				verror("Internal error: track %s of type %s is not supported by 1D iterators", itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
+			shared_ptr<GenomeTrack> new_track = init_1d_track_with_shared_backend(filename, interval.chromid, itrack_n_imdf->type);
+			if (!new_track) {
+				verror("Internal error: track %s of type %s is not supported by 1D iterators",
+					   itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
 			}
+			itrack_n_imdf->track = new_track;
 		}
 		catch (TGLException &e)
 		{
@@ -1484,6 +1573,10 @@ void TrackExpressionVars::start_chrom(const GInterval &interval)
 
 void TrackExpressionVars::start_chrom(const GInterval2D &interval)
 {
+	unordered_map<string, shared_ptr<GenomeTrack> > prev_shared_1d_track_masters;
+	prev_shared_1d_track_masters.swap(m_shared_1d_track_masters);
+	const bool shared_1d_backend_enabled = is_shared_1d_backend_enabled();
+
 	for (Track_n_imdfs::iterator itrack_n_imdf = m_track_n_imdfs.begin(); itrack_n_imdf != m_track_n_imdfs.end(); ++itrack_n_imdf) {
 		try {
 			if (GenomeTrack::is_1d(itrack_n_imdf->type)) {
@@ -1496,37 +1589,63 @@ void TrackExpressionVars::start_chrom(const GInterval2D &interval)
 				else
 					verror("Internal error: no 2D to 1D conversion for track %s", itrack_n_imdf->name.c_str());
 
-				if (chromid != itrack_n_imdf->imdf1d->interval.chromid) {
-					string track_dir = track2path(m_iu.get_env(), itrack_n_imdf->name);
-					string resolved = GenomeTrack::find_existing_1d_filename(m_iu.get_chromkey(), track_dir, chromid);
-					string filename(track_dir + "/" + resolved);
+				const bool track_supports_shared_backend = supports_shared_1d_backend(itrack_n_imdf->type);
+				const bool should_manage_shared_masters = track_supports_shared_backend &&
+					(shared_1d_backend_enabled || !prev_shared_1d_track_masters.empty() || !m_shared_1d_track_masters.empty());
 
-					delete itrack_n_imdf->track;
-					if (itrack_n_imdf->type == GenomeTrack::FIXED_BIN) {
-						itrack_n_imdf->track = new GenomeTrackFixedBin;
-						((GenomeTrackFixedBin *)itrack_n_imdf->track)->init_read(filename.c_str(), chromid);
-					} else if (itrack_n_imdf->type == GenomeTrack::SPARSE) {
-						itrack_n_imdf->track = new GenomeTrackSparse;
-						((GenomeTrackSparse *)itrack_n_imdf->track)->init_read(filename.c_str(), chromid);
-					} else if (itrack_n_imdf->type == GenomeTrack::ARRAYS) {
-						itrack_n_imdf->track = new GenomeTrackArrays;
-						((GenomeTrackArrays *)itrack_n_imdf->track)->init_read(filename.c_str(), chromid);
-					} else
-						verror("Internal error: track %s of type %s is not supported by 1D iterators (projected from 2D)", itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
+				string track_dir;
+				string resolved;
+				string filename;
+				if (chromid != itrack_n_imdf->imdf1d->interval.chromid || should_manage_shared_masters) {
+					track_dir = track2path(m_iu.get_env(), itrack_n_imdf->name);
+					resolved = GenomeTrack::find_existing_1d_filename(m_iu.get_chromkey(), track_dir, chromid);
+					filename = track_dir + "/" + resolved;
+				}
+
+				if (chromid != itrack_n_imdf->imdf1d->interval.chromid) {
+					shared_ptr<GenomeTrack> new_track = init_1d_track_with_shared_backend(filename, chromid, itrack_n_imdf->type);
+					if (!new_track) {
+						verror("Internal error: track %s of type %s is not supported by 1D iterators (projected from 2D)",
+							   itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
+					}
+					itrack_n_imdf->track = new_track;
+				} else if (should_manage_shared_masters) {
+					const string backend_key = shared_1d_backend_key(itrack_n_imdf->type, filename, chromid);
+					unordered_map<string, shared_ptr<GenomeTrack> >::iterator imaster =
+						m_shared_1d_track_masters.find(backend_key);
+
+					if (imaster != m_shared_1d_track_masters.end()) {
+						shared_ptr<GenomeTrack> new_track = create_dependent_1d_track(itrack_n_imdf->type, imaster->second);
+						itrack_n_imdf->track = new_track ? new_track : imaster->second;
+					} else {
+						unordered_map<string, shared_ptr<GenomeTrack> >::iterator iprev_master =
+							prev_shared_1d_track_masters.find(backend_key);
+						if (iprev_master != prev_shared_1d_track_masters.end()) {
+							m_shared_1d_track_masters.insert(make_pair(backend_key, iprev_master->second));
+						} else {
+							shared_ptr<GenomeTrack> new_track = init_1d_track_with_shared_backend(filename, chromid, itrack_n_imdf->type);
+							if (!new_track) {
+								verror("Internal error: track %s of type %s is not supported by 1D iterators (projected from 2D)",
+									   itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
+							}
+							itrack_n_imdf->track = new_track;
+						}
+					}
 				}
 			} else if (!m_interval2d.is_same_chrom(interval)) {
 				string filename(track2path(m_iu.get_env(), itrack_n_imdf->name) + "/" + GenomeTrack::get_2d_filename(m_iu.get_chromkey(), interval.chromid1(), interval.chromid2()));
-
-				delete itrack_n_imdf->track;
 				if (itrack_n_imdf->type == GenomeTrack::RECTS) {
-					itrack_n_imdf->track = new GenomeTrackRectsRects(m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
-					((GenomeTrackRectsRects *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					auto t = make_shared<GenomeTrackRectsRects>(m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
+					t->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					itrack_n_imdf->track = t;
 				} else if (itrack_n_imdf->type == GenomeTrack::POINTS) {
-					itrack_n_imdf->track = new GenomeTrackRectsPoints(m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
-					((GenomeTrackRectsPoints *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					auto t = make_shared<GenomeTrackRectsPoints>(m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
+					t->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					itrack_n_imdf->track = t;
 				} else if (itrack_n_imdf->type == GenomeTrack::COMPUTED) {
-					itrack_n_imdf->track = new GenomeTrackComputed(m_groot, m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
-					((GenomeTrackComputed *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					auto t = make_shared<GenomeTrackComputed>(m_groot, m_iu.get_track_chunk_size(), m_iu.get_track_num_chunks());
+					t->init_read(filename.c_str(), interval.chromid1(), interval.chromid2());
+					itrack_n_imdf->track = t;
 				} else
 					verror("Internal error: track %s of type %s is not supported by 2D iterators", itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
 			}
@@ -1582,30 +1701,22 @@ void TrackExpressionVars::set_vars(unsigned idx)
 	// Setup tracks (read intervals for non-sequence-based tracks)
 	for (Track_n_imdfs::iterator itrack_n_imdf = m_track_n_imdfs.begin(); itrack_n_imdf != m_track_n_imdfs.end(); ++itrack_n_imdf)
 	{
-		// Skip track setup for sequence-based tracks
-		Track_vars::iterator seq_var = m_track_vars.begin();
-		for (; seq_var != m_track_vars.end(); ++seq_var)
-		{
-			if (seq_var->track_n_imdf == &(*itrack_n_imdf) &&
-				TrackExpressionVars::is_sequence_based_function(seq_var->val_func))
-				break;
-		}
-		if (seq_var != m_track_vars.end())
+		if (itrack_n_imdf->has_only_sequence_functions)
 			continue;
 		try {
-			if (GenomeTrack::is_2d(itrack_n_imdf->type)) {
-				if (itrack_n_imdf->imdf2d) {
-					if (!itrack_n_imdf->imdf2d->out_of_range)
-						((GenomeTrack2D *)itrack_n_imdf->track)->read_interval(itrack_n_imdf->imdf2d->interval, m_band);
-				} else
-					((GenomeTrack2D *)itrack_n_imdf->track)->read_interval(m_interval2d, m_band);
-			} else {
-				if (itrack_n_imdf->imdf1d) {
-					if (!itrack_n_imdf->imdf1d->out_of_range)
-						((GenomeTrack1D *)itrack_n_imdf->track)->read_interval(itrack_n_imdf->imdf1d->interval);
-				} else
-					((GenomeTrack1D *)itrack_n_imdf->track)->read_interval(m_interval1d);
-			}
+				if (GenomeTrack::is_2d(itrack_n_imdf->type)) {
+					if (itrack_n_imdf->imdf2d) {
+						if (!itrack_n_imdf->imdf2d->out_of_range)
+							((GenomeTrack2D *)itrack_n_imdf->track.get())->read_interval(itrack_n_imdf->imdf2d->interval, m_band);
+					} else
+						((GenomeTrack2D *)itrack_n_imdf->track.get())->read_interval(m_interval2d, m_band);
+				} else {
+					if (itrack_n_imdf->imdf1d) {
+						if (!itrack_n_imdf->imdf1d->out_of_range)
+							((GenomeTrack1D *)itrack_n_imdf->track.get())->read_interval(itrack_n_imdf->imdf1d->interval);
+					} else
+						((GenomeTrack1D *)itrack_n_imdf->track.get())->read_interval(m_interval1d);
+				}
 		} catch (TGLException &e) {
 			verror("%s", e.msg());
 		}
