@@ -13,6 +13,7 @@
 #include <string>
 #include <string.h>
 #include <memory>
+#include <unordered_map>
 
 #ifndef R_NO_REMAP
 #  define R_NO_REMAP
@@ -98,13 +99,14 @@ public:
 
     struct Track_n_imdf {
         string               name;
-        GenomeTrack         *track{NULL};
+        std::shared_ptr<GenomeTrack> track;
         GenomeTrack::Type    type;
         vector<unsigned>     slice;
         GenomeTrackArrays::SliceFunctions slice_func;
         double               slice_percentile;
         Iterator_modifier1D *imdf1d{NULL};
         Iterator_modifier2D *imdf2d{NULL};
+        bool                 has_only_sequence_functions{false};
     };
 
     typedef vector<Track_n_imdf> Track_n_imdfs;
@@ -118,6 +120,7 @@ public:
             REG_NEAREST,
             STDDEV,
             SUM,
+            LSE,
             QUANTILE,
             PV,
             PV_MIN,
@@ -197,7 +200,7 @@ public:
 
     struct Value_var {
         enum Val_func {
-            AVG, MIN, MAX, STDDEV, SUM, QUANTILE,
+            AVG, MIN, MAX, STDDEV, SUM, LSE, QUANTILE,
             NEAREST,
             EXISTS, SIZE,
             FIRST, LAST, SAMPLE,
@@ -265,6 +268,7 @@ private:
 	GInterval               m_interval1d;
 	GInterval2D             m_interval2d;
 	DiagonalBand            m_band;
+	std::unordered_map<std::string, std::shared_ptr<GenomeTrack> > m_shared_1d_track_masters;
 
 	// Shared sequence fetcher for all sequence-based vtracks to enable caching
 	GenomeSeqFetch          m_shared_seqfetch;
@@ -290,6 +294,9 @@ private:
 	template<typename VarType>
 	void                 attach_filter_to_var(SEXP rvtrack, const string &vtrack, VarType &var);
 	void                 register_track_functions();
+	bool                 is_sequence_track_n_imdf(const Track_n_imdf &track_n_imdf) const;
+	void                 reset_shared_1d_track_masters();
+	std::shared_ptr<GenomeTrack> init_1d_track_with_shared_backend(const std::string &filename, int chromid, GenomeTrack::Type type);
 
 	void start_chrom(const GInterval &interval);
 	void start_chrom(const GInterval2D &interval);
